

import smbus2
import time

bus = smbus2.SMBus(1)

VDDCORE_ADDR = 0x40
VDDRCORE_ADDR = 0x41
DVDD33_ADDR = 0x4c
DVDD18_ADDR = 0x45
VDDA18_ADDR = 0x4e
VDDLPDDR_ADDR = 0x46

CONFIG_REG = 0x00
POWER_REG = 0x03
CALIB_REG = 0x05

#uA/bit
VDDCORE_CURRENT_LSB = 250 
VDDRCORE_CURRENT_LSB = 200 
DVDD33_CURRENT_LSB = 200
DVDD18_CURRENT_LSB = 200
VDDA18_CURRENT_LSB = 200 
VDDLPDDR_CURRENT_LSB = 200 

# ohms
VDDCORE_SHUNT = 0.01 
VDDRCORE_SHUNT = 0.01 
DVDD33_SHUNT = 0.01
DVDD18_SHUNT = 0.01
VDDA18_SHUNT = 0.01
VDDLPDDR_SHUNT = 0.01 


def configure_ina(device_addr):
    config_data = [0x49,0x27] #64 averages per read, otherwise defaul
    bus.write_i2c_block_data(device_addr, CONFIG_REG, config_data)

def write_calibration(addr, lsb, shunt):
    calib_value = int(1e6 * 0.00512 / (lsb * shunt))
    calib_data = [(calib_value >> 8) & 0xff, calib_value & 0xff]

    bus.write_i2c_block_data(addr, CALIB_REG, calib_data)


def setup():

    configure_ina(VDDCORE_ADDR)
    configure_ina(VDDRCORE_ADDR)
    configure_ina(DVDD33_ADDR)
    configure_ina(DVDD18_ADDR)
    configure_ina(VDDA18_ADDR)
    configure_ina(VDDLPDDR_ADDR)
    
    write_calibration(VDDCORE_ADDR, VDDCORE_CURRENT_LSB, VDDCORE_SHUNT)
    write_calibration(VDDRCORE_ADDR, VDDRCORE_CURRENT_LSB, VDDRCORE_SHUNT)
    write_calibration(DVDD33_ADDR, DVDD33_CURRENT_LSB, DVDD33_SHUNT)
    write_calibration(DVDD18_ADDR, DVDD18_CURRENT_LSB, DVDD18_SHUNT)
    write_calibration(VDDA18_ADDR, VDDA18_CURRENT_LSB, VDDA18_SHUNT)
    write_calibration(VDDLPDDR_ADDR, VDDLPDDR_CURRENT_LSB, VDDLPDDR_SHUNT)


def read_power(addr, lsb):
    power_data = bus.read_i2c_block_data(addr, POWER_REG, 2)
    # print(power_data)
    power_unconverted = (power_data[0] << 8) + power_data[1]
    power = lsb * 25 * power_unconverted

    power_W = power/1e6
    # print('Read power at %f W' % power_W)

    return power_W

def read_power_vddcore():
    return read_power(VDDCORE_ADDR, VDDCORE_CURRENT_LSB)

def read_power_vddrcore():
    return read_power(VDDRCORE_ADDR, VDDRCORE_CURRENT_LSB)

def read_power_dvdd33():
    return read_power(DVDD33_ADDR, DVDD33_CURRENT_LSB)

def read_power_dvdd18():
    return read_power(DVDD18_ADDR, DVDD18_CURRENT_LSB)

def read_power_vdda18():
    return read_power(VDDA18_ADDR, VDDA18_CURRENT_LSB)

def read_power_vddlpddr():
    return read_power(VDDLPDDR_ADDR, VDDLPDDR_CURRENT_LSB)

if __name__ == "__main__":
    setup()
    while True: 

        p_vddcore = read_power_vddcore()
        p_vddrcore = read_power_vddrcore()
        p_dvdd33 = read_power_dvdd33()
        p_dvdd18 = read_power_dvdd18()
        p_vdda18 = read_power_vdda18()
        p_vddlpddr = read_power_vddlpddr()

        p_total = p_vddcore+p_vddrcore+p_dvdd33+p_dvdd18+p_vdda18+p_vddlpddr
        print('total power: %f' % p_total)

        time.sleep(1)